#include <stdio.h>
#include <math.h>
#include "mkl.h"

//   
#define BIG_SEED      2000000000
//   ( )
#define BASE_GEN      VSL_BRNG_MCG59
//     
#define GAUSS_METHOD  VSL_METHOD_DGAUSSIAN_ICDF

//   
#define TIME      3.0
//  
#define NPATHS    50
//  
#define NSTEPS    300


//  
VSLStreamStatePtr InitGen()
{
  VSLStreamStatePtr stream;
  const unsigned int seed[2] = {BIG_SEED, BIG_SEED};
  //  
  vslNewStreamEx(&stream, BASE_GEN, 2, seed);
  
  return stream;
}

//    
void FreeGen(VSLStreamStatePtr stream)
{
  vslDeleteStream(&stream);
}

//    N(a, b)
//   z  count    
//  ,   
//      
// stream -       MKL
void GenerateGauss(double a, double b, int count, 
  VSLStreamStatePtr stream, double *z)
{
  // o count        z
  vdRngGaussian(GAUSS_METHOD, stream, count, z, a, b);
}

//    csv-
void PrintToFile(double **buffer, int nRows, int nColumns, char * FileName)
{
  FILE *f = fopen(FileName, "w");
  for (int i =0; i < nRows; i++)
  {
    for (int j = 0; j < nColumns; j++)
      fprintf(f, "%lf;", buffer[i][j]);
    fprintf(f, "\n");
  }
  fclose(f);
}

void SimulateWienerProcess(int nPaths, int nSteps, double Time, double **buffer)
{
  //  
  VSLStreamStatePtr stream = InitGen();
  //   
  double *dw = new double[nSteps];

  //   
  double h = Time / (double)nSteps;
  //   
  for (int i = 0; i < nPaths; i++)
  {
    //  ,   nSteps  
    GenerateGauss(0, sqrt(h), nSteps, stream, dw);
    buffer[i][0] = 0;
    //    
    for (int j = 1; j <= nSteps; j++)
    {
      buffer[i][j] = buffer[i][j - 1] + dw[j - 1];
    }
  }

  //  
  delete [] dw;
  //    
  FreeGen(stream);
}

int main()
{
  double **buffer = new double*[NPATHS];
  for (int i = 0; i < NPATHS; i++)
    buffer[i] = new double[NSTEPS + 1];

  SimulateWienerProcess(NPATHS, NSTEPS, TIME, buffer);
  PrintToFile(buffer, NPATHS, NSTEPS + 1, "results.csv");

  for (int i = 0; i < NPATHS; i++)
    delete [] buffer[i];
  delete [] buffer;

  return 0;
}
