#include <stdio.h>
#include <math.h>
#include "mkl.h"

//   
#define BIG_SEED      2000000000
//   ( )
#define BASE_GEN      VSL_BRNG_MCG59
//     
#define GAUSS_METHOD  VSL_METHOD_DGAUSSIAN_ICDF

//  
#define NPATHS    10000

//  
//#define NSTEPS    300

//  (%  0.2 -> 20%)
#define SIG   0.2
//   (%  0.05 -> -5%)
#define R     0.05
//    (3 -> 3  )
#define TIME  3.0
//      t=0 (..)
#define S0    100.0

//  
VSLStreamStatePtr InitGen()
{
  VSLStreamStatePtr stream;
  const unsigned int seed[2] = {BIG_SEED, BIG_SEED};
  //  
  vslNewStreamEx(&stream, BASE_GEN, 2, seed);
  return stream;
}

//    
void FreeGen(VSLStreamStatePtr stream)
{
  vslDeleteStream(&stream);
}

//    N(a, b)
//   z  count    
//  ,   
//      
// stream -       MKL
void GenerateGauss(double a, double b, int count, VSLStreamStatePtr stream, double *z)
{
  // o count        z
  vdRngGaussian(GAUSS_METHOD, stream, count, z, a, b);
}

//    csv-
void PrintToFile(double *buffer, int n, char * FileName)
{
  FILE *f = fopen(FileName, "w");
  for (int i =0; i < n; i++)
  {
    fprintf(f, "%lf;\n", buffer[i]);
  }
  fclose(f);
}

//    
// dS(t) = S(t) (r(t) dt + sig(t) dWt), S(0) > 0 ( S(0) = S0)

// S(t) -  
// W -   :
//    W(0) = 0
//    W(t) -    1
//    W(t) - W(s) -  N(0, t-s),  0 <= s < t
// r -  
// sig - 

//  r(t) = const, sig(t) = const   :
// S(Time) = S(0) * exp ((r - sig * sig / 2.0) * Time + sig * z),
//  z -   , z  N(0, Time)

//    S(t)
// z = Wt
double GetStockPrice(double z, double t)
{
  return S0 * exp( (R - SIG * SIG / 2.0) * t + SIG * z);
}

//    S(Time)  nPaths 
//    sBuffer,    
//     sBuffer
double SimulateStockPrices(int nPaths, double Time, double *sBuffer)
{
  //  
  VSLStreamStatePtr stream = InitGen();
  //      
  double *wBuffer = new double[nPaths];
  //  ,   nPaths  
  GenerateGauss(0, sqrt(Time), nPaths, stream, wBuffer);
  //  W(0)=0, W(Time) = dW,  W   W 
  //     nPaths  W(Time)

  double sum = 0;
  for (int i = 0; i < nPaths; i++)
  {
    sBuffer[i] = GetStockPrice(wBuffer[i], Time);
    sum += sBuffer[i];
  }
  sum = sum / (double)nPaths;

  FreeGen(stream);
  delete [] wBuffer;

  return sum;
}

int main()
{
  double *sBuffer = new double[NPATHS];

  double avg = SimulateStockPrices(NPATHS, TIME, sBuffer);
  PrintToFile(sBuffer, NPATHS, "results.csv");
  printf("Average price = %lf\n", avg);

  delete [] sBuffer;

  return 0;
}
